module SlidePay
  module ACH
    class Retrieval < Resource
      attr_accessor :retrieval_bank_account

      @url_root = "retrieval"
      @id_attribute = "retrieval_id"

      def initialize(values_hash={})
        @url_root = "retrieval"
        @id_attribute = "retrieval_id"

        # if values_hash[:retrieval_bank_account] is present and a hash, then we should probably pull
        # that out and set the instance attribute for bank account
        # @retrieval_bank_account = values_hash[:retrieval_bank_account]
        # values_hash.remove(:retrieval_bank_account)

        super(values_hash)
      end

      # url should allow retrieving by settlement_transaction_token
      def url
        if is_new?
          @url_root
        elsif self.settlement_transaction_token != nil
          "#{@url_root}?settlement_transaction_token=#{self['settlement_transaction_token']}"
        else
          "#{@url_root}/#{self.id()}"
        end
      end

      def settlement_transaction_token
        self['settlement_transaction_token']
      end

      def is_new?
        if id() == nil && self.settlement_transaction_token == nil
          true
        else
          false
        end
      end

      def create_with_bank_account(bank_account, options={})
        options[:token]     = options[:token] || @token
        options[:api_key]   = options[:api_key] || @api_key
        options[:endpoint]  = options[:endpoint] || @endpoint

        r = self.class.create_with_bank_account(bank_account, options)
        merge! r
      end

      def retrieve_by_settlement_token(settlement_transaction_token, options={})
        options[:token]     = options[:token] || @token
        options[:api_key]   = options[:api_key] || @api_key
        options[:endpoint]  = options[:endpoint] || @endpoint

        r = self.class.retrieve_by_settlement_token(settlement_transaction_token, options)
        merge! r
      end

      def is_ledgered?
        return self['ledgered'] == 1
      end

      def ledger_after
        return self['ledger_after']
      end

      def retrieval_amount
        return self['retrieval_amount']
      end

      def provider_status
        return self['provider_status']
      end

      class << self
        def create_with_bank_account(bank_account, options={})
          token = options[:token]
          api_key = options[:api_key]
          endpoint = options[:endpoint]

          response = SlidePay.post(path: "#{@url_root}/manual", data: bank_account.to_json, token: token, api_key: api_key, endpoint: endpoint)
          if response.was_successful?
            r = self.new()
            r.populate_from_response(response)
          end
        end

        def retrieve_by_settlement_token(settlement_transaction_token, options={})
          if ! settlement_transaction_token.is_a? String
            raise "settlement_transaction_token must be a String"
          end

          token = options[:token]
          api_key = options[:api_key]
          endpoint = options[:endpoint]

          url_suffix = "?settlement_transaction_token=#{settlement_transaction_token}"

          response = SlidePay.get(path: "#{@url_root}#{url_suffix}", token: token, api_key: api_key, endpoint: endpoint)
          if response.was_successful?
            r = self.new()
            r.populate_from_response(response)
          end
        end
      end
    end
  end
end
