module SlidePay
  module ACH
    class Settlement < Resource
      @url_root = "settlement"
      @id_attribute = "settlement_id"

      def initialize(values_hash={})
        @url_root = "settlement"
        @id_attribute = "settlement_id"

        super(values_hash)
      end

      class << self

        def process(settlement_order, options={})
          validate_settlement_order(settlement_order)

          token = @token || options[:token]
          api_key = @api_key || options[:api_key]
          endpoint = @endpoint || options[:endpoint]

          response = SlidePay.post(path: "settlement/manual", api_key: api_key, token: token, endpoint: endpoint, data: settlement_order.to_json)

          if response.was_successful?
            return response.data
          elsif response.error_text
            raise Exception.new(response.error_text)
          elsif response.data["status_message"]
            raise Exception.new(response.data["status_message"])
          else
            raise Exception.new("Settlement(s) could not be processed.")
          end
        end

        private

        def validate_single_settlement_descriptor(descriptor)
          puts "Descriptor: #{descriptor}\n\n"
          if ! descriptor.is_a? Hash
            raise "Cannot validate a settlement descriptor that is not a hash."
          elsif ! descriptor.include?(:bank_account_id) && ! descriptor.include?("bank_account_id")
            raise "A bank_account_id is required for each settlement being processed."
          elsif ! descriptor.include?(:amount) && ! descriptor.include?("amount")
            raise "An amount is required for each settlement being processed."
          else
            return true
          end
        end

        def validate_settlement_order(settlement_order)
          raise "A settlement must be a Hash object." unless settlement_order.is_a? Hash
          raise "company_id is required for a settlement." unless (settlement_order['company_id'] || settlement_order[:company_id])
          raise "location_id is required for a settlement." unless (settlement_order['location_id'] || settlement_order[:location_id])
          raise "settlement_list is required for a settlement." unless (settlement_order['settlement_list'] || settlement_order[:settlement_list])

          descriptor_list = settlement_order['settlement_list'] || settlement_order[:settlement_list]
          descriptor_list.each do |descriptor|
            validate_single_settlement_descriptor(descriptor)
          end

          return true
        end

      end
    end
  end
end