module SlidePay
  class ApiResource < Hash
    attr_accessor :url_root, :id_attribute, :token, :api_key, :endpoint

    def initialize(values_hash={})
      values_clone = values_hash.clone()

      if values_clone[:url_root]
        @url_root = values_clone[:url_root]
        values_clone.delete(:url_root)
      end

      if values_clone[:id_attribute]
        @id_attribute = values_clone[:id_attribute]
        values_clone.delete(:id_attribute)
      end

      if values_clone[:token]
        @token = values_clone[:token]
        values_clone.delete(:token)
      end

      if values_clone[:api_key]
        @api_key = values_clone[:api_key]
        values_clone.delete(:api_key)
      end

      if values_clone[:endpoint]
        @endpoint = values_clone[:endpoint]
        values_clone.delete(:endpoint)
      end

      merge! values_clone
    end

    def url
      if is_new?
        @url_root
      else
        "#{@url_root}/#{self.id()}"
      end
    end

    def id
      self[@id_attribute]
    end

    def is_new?
      if id() == nil
        true
      else
        false
      end
    end

    def populate_from_response(response)
      if response.data.instance_of? Array
        self.merge! response.data.first
      elsif response.data.instance_of? Hash
        self.merge! response.data
      else
        raise Exception.new("Resource with the specified id not found on the server.")
      end
    end

    def retrieve(options_hash={})
      token = @token || options_hash[:token]
      api_key = @api_key || options_hash[:api_key]
      endpoint = @endpoint || options_hash[:endpoint]

      if is_new?
        raise Exception.new("Cannot retrieve an unsaved object from the server.")
      end

      response = SlidePay.get(path: self.url(), token: token, api_key: api_key, endpoint: endpoint)
      if response.was_successful?
        self.populate_from_response(response)
      end
    end

    def save(options_hash={})
      token = @token || options_hash[:token]
      api_key = @api_key || options_hash[:api_key]
      endpoint = @endpoint || options_hash[:endpoint]

      if is_new?
        response = SlidePay.post(path: self.url(), token: token, api_key: api_key, endpoint: endpoint, data: self.to_json)
      else
        response = SlidePay.put(path: self.url(), token: token, api_key: api_key, endpoint: endpoint, data: self.to_json)
      end

      if response.was_successful?
        self.populate_from_response(response)
        true
      else
        false
      end
    end

    def destroy(options_hash={})
      token = @token || options_hash[:token]
      api_key = @api_key || options_hash[:api_key]
      endpoint = @endpoint || options_hash[:endpoint]

      if is_new?
        raise Exception.new("Cannot destroy a resource that has not been saved.")
      end

      response = SlidePay.delete(path: self.url(), token: token, api_key: api_key, endpoint: endpoint)

      if response.was_successful?
        self[@id_attribute] = nil
        true
      else
        false
      end
    end

    class << self
      attr_accessor :url_root, :id_attribute

      def api_resource_array_from_response(response)
        resources_array = []
        if response.was_successful? and response.data != nil
          data = response.data
          data.each do |resource_hash|
            resources_array.push self.new(resource_hash)
          end
        end

        resources_array
      end

      def retrieve(token=nil, api_key=nil, endpoint=nil)
        response = SlidePay.get(path: @url_root, token: token, api_key: api_key, endpoint: endpoint)

        return api_resource_array_from_response(response)
      end

      def find(search_params, token=nil, api_key=nil, endpoint=nil)
        if search_params.is_a? Hash
          search_params = [search_params]
        elsif not search_params.is_a? Array
          raise Exception.new("Invalid or no search filter supplied.")
        end
        response = SlidePay.put(path: @url_root, data: search_params.to_json, token: token, api_key: api_key, endpoint: endpoint)

        return api_resource_array_from_response(response)
      end

      def find_where(search_params, token=nil, api_key=nil, endpoint=nil)
        sfa = []
        search_params.each_pair do |k,v|
          sfa.push(field: k, condition: "equals", value: v)
        end

        return find(sfa, token, api_key, endpoint)
      end

      def find_between(start_params, end_params, token=nil, api_key=nil, endpoint=nil)
        sfa = []
        start_params.each_pair do |k,v|
          sfa.push(field: k, condition: "greater_than", value: v)
        end

        end_params.each_pair do |k,v|
          sfa.push(field: k, condition: "less_than", value: v)
        end

        return find(sfa, token, api_key, endpoint)
      end

      def find_greater_than(search_params, token=nil, api_key=nil, endpoint=nil)
        sfa = []
        search_params.each_pair do |k,v|
          sfa.push(field: k, condition: "greater_than", value: v)
        end

        return find(sfa, token, api_key, endpoint)
      end

      def find_less_than(search_params, token=nil, api_key=nil, endpoint=nil)
        sfa = []
        search_params.each_pair do |k,v|
          sfa.push(field: k, condition: "less_than", value: v)
        end

        return find(sfa, token, api_key, endpoint)
      end
    end

  end
end
